local isInputOutputEmpty = function(inputOrOutput)
	if inputOrOutput == nil then
		return true
	elseif next(inputOrOutput) == nil then
		return true
	else
		for num, sub in pairs(inputOrOutput) do
			if num ~= 1 then
				return false
			elseif type(sub) == "table" then
				if next(sub) ~= nil then
					return false
				end
			else
				return false
			end
		end
		return true
	end
end


local function constructionCallback(fileName, data)
	local updateFn = data.updateFn
	if data.type == "INDUSTRY" then
		if data.updateFn ~= nil then
			data.updateFn = function(params)
				local result = updateFn(params)
				if result ~= nil and result.rule ~= nil then
					if result.rule.capacity ~= nil then
						local source = isInputOutputEmpty(result.rule.input)
						if source == true then
							if isInputOutputEmpty(result.rule.output) == false then
								-- increase production of source industries
								result.rule.capacity = result.rule.capacity * 1.5
							end
						end
					end
				end
				return result
			end
		end
	end
	return data
end


function data()
return {
	info = {
		minorVersion = 0,
		severityAdd = "NONE",
		severityRemove = "WARNING",
		name = _("Industry Expanded"),
		description = _("Introduces 13 new cargo types and many new industries to make for a more complex, yet balanced experience. Appropriate cargo vehicles support the new cargo types. This mod does not overwrite any vanilla assets, so it is 100% compatible with any other industry or cargo demand mod!"),
		tags = { "Script Mod", "Industry", "Misc", "Resource" },
		authors = {
			{
				name = "Col0Korn",
				role = "CREATOR",
			},
		},
	},
	
	runFn = function (settings)
		addModifier("loadConstruction", constructionCallback)
		addModifier("loadModel", function (fileName, data)
			if (string.find(fileName, "vehicle") == nil) then
				return data
			end
			if (data.metadata) then
				if (data.metadata.transportVehicle) then
					if (data.metadata.transportVehicle.carrier) then
						if(data.metadata.transportVehicle.compartments) then
							for i=1,#data.metadata.transportVehicle.compartments do
								local foundFoodCapacity  = 0
								local foundFoodCargobay  = null
								local foundPlasticCapacity  = 0
								local foundPlasticCargobay  = null
								local foundConstructionCapacity  = 0
								local foundConstructionCargobay  = null
								local foundLogsCapacity  = 0
								local foundLogsCargobay  = null
								local foundCoalCapacity  = 0
								local foundCoalCargobay  = null
								for j=1,#data.metadata.transportVehicle.compartments[i] do
									for k=1,#data.metadata.transportVehicle.compartments[i][j] do
										if (string.upper(data.metadata.transportVehicle.compartments[i][j][k].type) == "FOOD") then
											foundFoodCapacity = data.metadata.transportVehicle.compartments[i][j][k].capacity
											foundFoodCargobay = data.metadata.transportVehicle.compartments[i][j][k].cargoBay
										elseif (string.upper(data.metadata.transportVehicle.compartments[i][j][k].type) == "PLASTIC") then
											foundPlasticCapacity = data.metadata.transportVehicle.compartments[i][j][k].capacity
											foundPlasticCargobay = data.metadata.transportVehicle.compartments[i][j][k].cargoBay
										elseif (string.upper(data.metadata.transportVehicle.compartments[i][j][k].type) == "CONSTRUCTION_MATERIALS") then
											foundConstructionCapacity = data.metadata.transportVehicle.compartments[i][j][k].capacity
											foundConstructionCargobay = data.metadata.transportVehicle.compartments[i][j][k].cargoBay
										elseif (string.upper(data.metadata.transportVehicle.compartments[i][j][k].type) == "LOGS") then
											foundLogsCapacity = data.metadata.transportVehicle.compartments[i][j][k].capacity
											foundLogsCargobay = data.metadata.transportVehicle.compartments[i][j][k].cargoBay
										elseif (string.upper(data.metadata.transportVehicle.compartments[i][j][k].type) == "COAL") then
											foundCoalCapacity = data.metadata.transportVehicle.compartments[i][j][k].capacity
											foundCoalCargobay = data.metadata.transportVehicle.compartments[i][j][k].cargoBay
										end
									end
								end
								if(foundFoodCapacity>0) then
									print("mod \"Industry Expanded\" modifying vehicle to carry Livestock: " .. fileName)
									table.insert(data.metadata.transportVehicle.compartments[i], { { type = "LIVESTOCK", capacity = foundFoodCapacity, cargoBay = foundFoodCargobay } })
									table.insert(data.metadata.transportVehicle.compartments[i], { { type = "FISH", capacity = foundFoodCapacity, cargoBay = foundFoodCargobay } })
									table.insert(data.metadata.transportVehicle.compartments[i], { { type = "MEAT", capacity = foundFoodCapacity, cargoBay = foundFoodCargobay } })
									table.insert(data.metadata.transportVehicle.compartments[i], { { type = "COFFEE_BERRIES", capacity = foundFoodCapacity, cargoBay = foundFoodCargobay } })
									table.insert(data.metadata.transportVehicle.compartments[i], { { type = "COFFEE", capacity = foundFoodCapacity, cargoBay = foundFoodCargobay } })
									table.insert(data.metadata.transportVehicle.compartments[i], { { type = "ALCOHOL", capacity = foundFoodCapacity, cargoBay = foundFoodCargobay } })
								end
								if(foundPlasticCapacity>0) then
									print("mod \"Industry Expanded\" modifying vehicle to carry Paper: " .. fileName)
									table.insert(data.metadata.transportVehicle.compartments[i], { { type = "PAPER", capacity = foundPlasticCapacity, cargoBay = foundPlasticCargobay } })
								end
								if(foundConstructionCapacity>0) then
									print("mod \"Industry Expanded\" modifying vehicle to carry Oil Sand: " .. fileName)
									table.insert(data.metadata.transportVehicle.compartments[i], { { type = "OIL_SAND", capacity = foundConstructionCapacity, cargoBay = foundConstructionCargobay } })
								end
								if(foundLogsCapacity>0) then
									print("mod \"Industry Expanded\" modifying vehicle to carry Marble: " .. fileName)
									table.insert(data.metadata.transportVehicle.compartments[i], { { type = "MARBLE", capacity = foundLogsCapacity, cargoBay = foundLogsCargobay } })
									table.insert(data.metadata.transportVehicle.compartments[i], { { type = "SILVER", capacity = foundLogsCapacity, cargoBay = foundLogsCargobay } })
								end
								if(foundCoalCapacity>0) then
									print("mod \"Industry Expanded\" modifying vehicle to carry Slag: " .. fileName)
									table.insert(data.metadata.transportVehicle.compartments[i], { { type = "SLAG", capacity = foundCoalCapacity, cargoBay = foundCoalCargobay } })
									table.insert(data.metadata.transportVehicle.compartments[i], { { type = "SAND", capacity = foundCoalCapacity, cargoBay = foundCoalCargobay } })
									table.insert(data.metadata.transportVehicle.compartments[i], { { type = "SILVER_ORE", capacity = foundCoalCapacity, cargoBay = foundCoalCargobay } })
								end
							end
						elseif(data.metadata.transportVehicle.compartmentsList) then
							for h=1,#data.metadata.transportVehicle.compartmentsList do
								if ( data.metadata.transportVehicle.compartmentsList[h].loadConfigs ) then
									for i=1,#data.metadata.transportVehicle.compartmentsList[h].loadConfigs do
										local foundFoodCapacity  = 0
										local foundFoodCargobay  = null
										local foundPlasticCapacity  = 0
										local foundPlasticCargobay  = null
										local foundConstructionCapacity  = 0
										local foundConstructionCargobay  = null
										local foundLogsCapacity  = 0
										local foundLogsCargobay  = null
										local foundCoalCapacity  = 0
										local foundCoalCargobay  = null
										-- Specific check/fix added for the mod: "modwerkstatt_speedlimits_1"
										if ( data.metadata.transportVehicle.compartmentsList[h].loadConfigs[i].cargoEntries ) then
											for j=1,#data.metadata.transportVehicle.compartmentsList[h].loadConfigs[i].cargoEntries do
												if (string.upper(data.metadata.transportVehicle.compartmentsList[h].loadConfigs[i].cargoEntries[j].type) == "FOOD") then
													foundFoodCapacity = data.metadata.transportVehicle.compartmentsList[h].loadConfigs[i].cargoEntries[j].capacity
													foundFoodCargobay = data.metadata.transportVehicle.compartmentsList[h].loadConfigs[i].cargoEntries[j].cargoBay
												elseif (string.upper(data.metadata.transportVehicle.compartmentsList[h].loadConfigs[i].cargoEntries[j].type) == "PLASTIC") then
													foundPlasticCapacity = data.metadata.transportVehicle.compartmentsList[h].loadConfigs[i].cargoEntries[j].capacity
													foundPlasticCargobay = data.metadata.transportVehicle.compartmentsList[h].loadConfigs[i].cargoEntries[j].cargoBay
												elseif (string.upper(data.metadata.transportVehicle.compartmentsList[h].loadConfigs[i].cargoEntries[j].type) == "CONSTRUCTION_MATERIALS") then
													foundConstructionCapacity = data.metadata.transportVehicle.compartmentsList[h].loadConfigs[i].cargoEntries[j].capacity
													foundConstructionCargobay = data.metadata.transportVehicle.compartmentsList[h].loadConfigs[i].cargoEntries[j].cargoBay
												elseif (string.upper(data.metadata.transportVehicle.compartmentsList[h].loadConfigs[i].cargoEntries[j].type) == "LOGS") then
													foundLogsCapacity = data.metadata.transportVehicle.compartmentsList[h].loadConfigs[i].cargoEntries[j].capacity
													foundLogsCargobay = data.metadata.transportVehicle.compartmentsList[h].loadConfigs[i].cargoEntries[j].cargoBay
												elseif (string.upper(data.metadata.transportVehicle.compartmentsList[h].loadConfigs[i].cargoEntries[j].type) == "COAL") then
													foundCoalCapacity = data.metadata.transportVehicle.compartmentsList[h].loadConfigs[i].cargoEntries[j].capacity
													foundCoalCargobay = data.metadata.transportVehicle.compartmentsList[h].loadConfigs[i].cargoEntries[j].cargoBay
												end
											end
											if(foundFoodCapacity>0) then
												print("mod \"Industry Expanded\" modifying vehicle to carry Livestock: " .. fileName)
												table.insert(data.metadata.transportVehicle.compartmentsList[h].loadConfigs, {cargoEntries = { { type = "LIVESTOCK", capacity = foundFoodCapacity, cargoBay = foundFoodCargobay }}})
												table.insert(data.metadata.transportVehicle.compartmentsList[h].loadConfigs, {cargoEntries = { { type = "FISH", capacity = foundFoodCapacity, cargoBay = foundFoodCargobay }}})
												table.insert(data.metadata.transportVehicle.compartmentsList[h].loadConfigs, {cargoEntries = { { type = "MEAT", capacity = foundFoodCapacity, cargoBay = foundFoodCargobay }}})
												table.insert(data.metadata.transportVehicle.compartmentsList[h].loadConfigs, {cargoEntries = { { type = "COFFEE_BERRIES", capacity = foundFoodCapacity, cargoBay = foundFoodCargobay }}})
												table.insert(data.metadata.transportVehicle.compartmentsList[h].loadConfigs, {cargoEntries = { { type = "COFFEE", capacity = foundFoodCapacity, cargoBay = foundFoodCargobay }}})
												table.insert(data.metadata.transportVehicle.compartmentsList[h].loadConfigs, {cargoEntries = { { type = "ALCOHOL", capacity = foundFoodCapacity, cargoBay = foundFoodCargobay }}})
											end
											if(foundPlasticCapacity>0) then
												print("mod \"Industry Expanded\" modifying vehicle to carry Paper: " .. fileName)
												table.insert(data.metadata.transportVehicle.compartmentsList[h].loadConfigs, {cargoEntries = { { type = "PAPER", capacity = foundPlasticCapacity, cargoBay = foundPlasticCargobay }}})
											end
											if(foundConstructionCapacity>0) then
												print("mod \"Industry Expanded\" modifying vehicle to carry Oil Sand: " .. fileName)
												table.insert(data.metadata.transportVehicle.compartmentsList[h].loadConfigs, {cargoEntries = { { type = "OIL_SAND", capacity = foundConstructionCapacity, cargoBay = foundConstructionCargobay }}})
											end
											if(foundLogsCapacity>0) then
												print("mod \"Industry Expanded\" modifying vehicle to carry Marble: " .. fileName)
												table.insert(data.metadata.transportVehicle.compartmentsList[h].loadConfigs, {cargoEntries = { { type = "MARBLE", capacity = foundLogsCapacity, cargoBay = foundLogsCargobay }}})
												table.insert(data.metadata.transportVehicle.compartmentsList[h].loadConfigs, {cargoEntries = { { type = "SILVER", capacity = foundLogsCapacity, cargoBay = foundLogsCargobay }}})
											end
											if(foundCoalCapacity>0) then
												print("mod \"Industry Expanded\" modifying vehicle to carry Slag: " .. fileName)
												table.insert(data.metadata.transportVehicle.compartmentsList[h].loadConfigs, {cargoEntries = { { type = "SLAG", capacity = foundCoalCapacity, cargoBay = foundCoalCargobay }}})
												table.insert(data.metadata.transportVehicle.compartmentsList[h].loadConfigs, {cargoEntries = { { type = "SAND", capacity = foundCoalCapacity, cargoBay = foundCoalCargobay }}})
												table.insert(data.metadata.transportVehicle.compartmentsList[h].loadConfigs, {cargoEntries = { { type = "SILVER_ORE", capacity = foundCoalCapacity, cargoBay = foundCoalCargobay }}})
											end
										end
									end
								end
							end
						end
					end
				end
			end
			return data
		end)
	end
	}
end